const ExtraSettings = preload("ExtraSettings.gd")
const TitleMenu = preload("res://menus/title/TitleMenu.gd")
const SettingsMenu = preload("res://menus/settings/SettingsMenu.gd")
const GameplayPanel = preload("res://menus/settings/GameplayPanel.gd")
const ArrowOptionListScene = preload("res://nodes/menus/ArrowOptionList.tscn")

class AddedUI:
	var icons : Control

func add_hook():
	yield (Co.pass(), "completed")
	MenuHelper.connect("top_menu_changed", self, "_on_top_menu_changed")
	SceneManager.connect("scene_change_ending", self, "_on_scene_change_ending")

func _on_top_menu_changed(menu):
	if menu != null && menu is SettingsMenu:
		var save = _find_save_button(menu)
		var gameplay = _find_gameplay_panel(menu)
		var inputs = _find_panel_inputs(gameplay) if gameplay != null else null

		if inputs == null || save == null || menu.defaults_button == null:
			return

		for child in inputs.get_children():
			if child is Label && child.text == "AP7_ICON_OVERRIDE_LABEL":
				return

		print("[IconOverride] Adding settings hook")
		var ui = AddedUI.new()
		ui.icons = _add_arrow_option(gameplay.inputs, "AP7_ICON_OVERRIDE_LABEL", \
			["", "sony", "xbox_one", "switch"], \
			["AP7_ICON_OVERRIDE_OPT_DEFAULT", "AP7_ICON_OVERRIDE_OPT_PS", "AP7_ICON_OVERRIDE_OPT_XBOX", "AP7_ICON_OVERRIDE_OPT_SWITCH"])
		gameplay.inputs.setup_focus()

		yield (Co.pass(), "completed") # UI needs to be ready before we can set initial values...
		ui.icons.set_selected_value(ExtraSettings.get_icons_setting())

		save.connect("pressed", self, "_on_save_pressed", [ui])
		menu.defaults_button.connect("pressed", self, "_on_defaults_pressed", [ui])

func _on_scene_change_ending():
	# The title menu does it's own stuff about creating and showing a settings menu, so we have to hook it a very hacky way :(
	print("[IconOverride] Adding title hook")
	var hooked_menu = false
	while SceneManager.current_scene is TitleMenu:
		if SceneManager.current_scene.menu != null && SceneManager.current_scene.menu is SettingsMenu:
			yield (Co.pass(), "completed")
			if !hooked_menu:
				hooked_menu = true
				_on_top_menu_changed(SceneManager.current_scene.menu)
		else:
			yield (Co.pass(), "completed")
			hooked_menu = false

	print("[IconOverride] Removing title hook")

func _on_save_pressed(ui: AddedUI):
	print("[IconOverride] Saving settings")
	ExtraSettings.set_icons_setting(ui.icons.selected_value)
	UserSettings.apply_settings()
	UserSettings.save_settings()

func _on_defaults_pressed(ui: AddedUI):
	ui.icons.set_selected_value("")

func _find_save_button(menu: SettingsMenu) -> Button:
	var node = menu.find_node("SaveButton")
	if node == null:
		print("[IconOverride] Setting menu was opened, but was unable to locate the save button")

	return node

func _find_gameplay_panel(menu: SettingsMenu) -> GameplayPanel:
	for content in menu.content_container.get_children():
		if content is GameplayPanel:
			return content

	print("[IconOverride] Setting menu was opened, but was unable to locate the gameplay panel")
	return null

func _find_panel_inputs(panel: Control) -> Control:
	if "inputs" in panel && panel.inputs != null:
		return panel.inputs
	
	print("[IconOverride] Panel was located, but couldn't add inputs")
	return null

func _add_arrow_option(inputs: Control, text: String, values: Array, value_labels: Array) -> Control:
	var label = Label.new()
	label.text = text
	label.add_color_override("font_color", Color.black)
	label.size_flags_horizontal |= Control.SIZE_EXPAND_FILL
	inputs.add_child(label)

	var input = ArrowOptionListScene.instance()
	input.values = values
	input.value_labels = value_labels
	input.size_flags_horizontal |= Control.SIZE_EXPAND_FILL
	inputs.add_child(input)
	return input
